.\" @(#)pic2lnt.l INRIA Rhone-Alpes - 12/10/12
.TH PIC2LNT LOCAL "12/10/12" "(C) INRIA" "CADP MANUAL PAGES"
.fp 4 CB
.lg 0
.SH NAME
pic2lnt \- translator of the applied pi-calculus to LOTOS NT

.SH SYNOPSIS
.B pic2lnt
[\fB\-pidlist\fR]
[\fB\-root\fR \fIinstantiation\fR]
[\fB\-silent\fR | \fB\-verbose\fR] 
[\fB\-version\fR]
\fIfilename\fP[\fB.pic\fP]

.SH DESCRIPTION
The pi-calculus [MPW92] is a specification formalism for concurrent
processes involving mobile communication. LOTOS NT is an imperatively
styled specification language for concurrent processes. 
The applied pi-calculus (hereafter noted PIC) is an extension
of the polyadic pi-calculus with the data types, functions, and
expressions of LOTOS NT.
The \fBpic2lnt\fP program translates a PIC specification into a LOTOS NT
specification. The translation implemented by \fBpic2lnt\fP generalizes the
approach proposed in [MS10] for translating polyadic pi-calculus into LOTOS NT.
.P
The input to \fBpic2lnt\fP is a PIC file, which must have the
extension \f4.pic\fR. If the user does not specify the
extension \f4.pic\fR on the command line, it will be appended
automatically and \fBpic2lnt\fP will read \fIfilename\fP\f4.pic\fR as input.
.P
The data types and functions used in a PIC specification must be defined
in external LOTOS NT files \f4.lib\fR, which must be included in the PIC
file (see SPECIFICATION below).
.P
The output of \fBpic2lnt\fP is a LOTOS NT specification.
The name of the input file is used to construct the name of the output file.
For an input file \f4example.pic\fR, \fBpic2lnt\fP creates the LOTOS NT 
specification \f4example.lnt\fR.

.SH OPTIONS
.TP
.B \-pidlist
List the names of all processes without parameters that occur in the
input file and exit. In other words, list all processes that can be used
as main process (by instantiation with the \fB\-root\fR option).
.TP
.BI \-root \ instantiation
Use the process \fIinstantiation\fP as the main process, meaning that the
generated LOTOS NT specification has the behaviour of the main process. The
\fIinstantiation\fP is the identifier of a PIC process having no parameters.
If this option is absent, the PIC specification must contain a process without
parameters, named MAIN (letter case is not relevant), which is the default
main process.
.TP
.B \-version
Display the tool version and exit.

.SH APPLIED PI-CALCULUS

The PIC language is an extension of the polyadic pi-calculus with the
data types, functions, and expressions of LOTOS NT.
.P
An overview of PIC is given below.
The abstract syntax of PIC is defined by an extended BNF (EBNF) grammar
and the semantics of each PIC construct is defined informally.
The EBNF grammar uses the following conventions:
PIC keywords are enclosed between double quotes;
a list of \fIn\fP elements is noted \fIELEM1\fP ... \fIELEMn\fP;
a list indexed from 0 (e.g., \fIX0\fP, ..., \fIXn\fP) has at least one element;
a list indexed from 1 (e.g., \fIX1\fP, ..., \fIXn\fP) may have 0 elements;
optional constructs are enclosed between square brackets.
.P
The symbols of the PIC grammar are listed in the table below.
The axiom of the grammar is the \fISPEC\fP symbol.
.P
.cs 4 23
.cs I 23
.nf
\f4             +--------+---------------------------------+
             | Symbol |           Description           |
  +----------+--------+---------------------------------+
  |          |  \fISPEC\fP  | specification                   |
  | Non-     |  \fIPROC\fP  | process definition              |
  | terminal |  \fIDECL\fP  | channel or variable declaration |
  |          |  \fIB\fP     | behaviour                       |
  +----------+--------+---------------------------------+
  |          |  \fIM\fP     | LOTOS NT module                 |
  |          |  \fIC\fP     | channel                         |
  |          |  \fIX\fP     | variable                        |
  | Terminal |  \fIF\fP     | function                        |
  |          |  \fIOP\fP    | operator                        |
  |          |  \fIT\fP     | type                            |
  |          |  \fIK\fP     | constant                        |
  +----------+--------+---------------------------------+\fR
.fi
.cs 4
.cs I
.P
Identifiers are built from letters, digits, and underscores (beginning
with a letter or an underscore). Keywords must be written in lowercase.
Comments are enclosed between "\f4(*\fP" and "\f4*)\fP" and may not be
nested. \fBpic2lnt\fP is case-sensitive.

.SS VALUE EXPRESSIONS

The syntax of PIC value expressions is defined by the following grammar:
.P
.cs 4 23
.cs I 23
.nf
\f4    \fIE\fP ::= \fIC\fP

       |  \fIX\fP

       |  \fIK\fP

       |  \fIF\fP [ "(" \fIE1\fP "," ... "," \fIEn\fP ")" ]

       |  \fIOP\fP \fIE\fP

       |  \fIE1\fP \fIOP\fP \fIE2\fP

       |  \fIE\fP "of" \fIT\fP

       |  \fIE\fP "." \fIX\fP

       |  \fIE\fP "." "{" \fIX0\fP "=>" \fIE0\fP, ..., \fIXn\fP "=>" \fIEn\fP "}"

       |  \fIE1\fP "[" \fIE2\fP "]"

       |  "(" \fIE\fP ")"\fR
.fi
.cs 4
.cs I
.P
The semantics of PIC value expressions is defined informally below.
.P
.cs R
.cs I
.nf
\fIC\fP
.fi
.cs R
.cs I
.IP  \" nbsp #160 (pour Rman)
is an occurrence of a channel name \fIC\fP. As in standard pi-calculus,
channel occurrences can be either \fIbound\fP (i.e., corresponding to channel
names \fIC\fP defined by an enclosing reception or a "new" operator, see
BEHAVIOURS below) or \fIfree\fP.
.sp
Channel names \fIC\fP are equipped with the standard comparison operators
"\f4=\fP" and "\f4<>\fP".
.P
.cs R
.cs I
.nf
\fIX\fP
.fi
.cs R
.cs I
.IP  \" nbsp #160 (pour Rman)
is a use occurrence of a data variable previously defined in the PIC
specification.
.P
.cs R
.cs I
.nf
\fIK\fP
.fi
.cs R
.cs I
.IP  \" nbsp #160 (pour Rman)
is a constant. The syntax of numerical, character, and string constants
in PIC is identical to that of LOTOS NT (e.g., \f413\fP, \f4-1\fP,
\f41.618\fP, \f4'a'\fP, \f4'\e007'\fP, \f4'\en'\fP, \f4"hello world\en"\fP).
.P
.cs R
.cs I
.nf
\fIF\fP [ "(" \fIE1\fP "," ... "," \fIEn\fP ")" ]
.fi
.cs R
.cs I
.IP  \" nbsp #160 (pour Rman)
is the call of function \fIF\fP having as arguments the values of expressions
\fIE1\fP, ..., \fIEn\fP. The arguments must correspond in number and types
to the formal parameters given in the definition of \fIF\fP (which must be
present in some external LOTOS NT module, see SPECIFICATION below).
Function calls with no arguments can be written either without parentheses
(e.g., \fIF\fP), or with parentheses (e.g., \fIF\fP ()).
.sp
Note: 
If a name \fIF\fP without parentheses occurs in an expression, two cases
are possible:
.sp
- If the name \fIF\fP does not correspond to a data variable previously
declared, then \fIF\fP is interpreted as the free occurrence of a channel.
If the name \fIF\fP denotes a function without arguments defined in some
LOTOS NT library, then it must be written using parentheses (i.e., \fIF\fP ())
in order to avoid its interpretation as a free channel occurrence.
.sp
-
If the name \fIF\fP corresponds both to a data variable \fIand\fP to a bound
channel previously declared, then \fIF\fP is interpreted as a use occurrence
of the data variable. If the name \fIF\fP must be interpreted as a channel
name, then the name of the channel must be changed in order to avoid the
clash with the name of the data variable.
.P
.cs R
.cs I
.nf
\fIOP\fP \fIE\fP

\fIE1\fP \fIOP\fP \fIE2\fP
.fi
.cs R
.cs I
.IP  \" nbsp #160 (pour Rman)
are calls to unary (prefixed) and binary (infixed) operators, respectively.
Standard arithmetic and relational operators are allowed
(e.g., "\f4-\fP", "\f4+\fP", "\f4*\fP", "\f4<=\fP", etc.).
.sp
Syntactically, all binary operators are left-associative and have the same
precedence (hence, one should explicitly add parentheses around subexpressions
to ensure a desired evaluation order, e.g., write "\f4x + (y * z)\fP" instead
of "\f4x + y * z\fP", which would be evaluated by default as
"\f4(x + y) * z\fP").
All unary operators have the same precedence, which is higher than the
precedence of binary operators.
.P
.cs R
.cs I
.nf
\fIE\fP "of" \fIT\fP
.fi
.cs R
.cs I
.IP  \" nbsp #160 (pour Rman)
is the explicit typing operator, which indicates that expression \fIE\fP is
of type \fIT\fP. This operator is useful for disambiguating certain value
expressions containing overloaded LOTOS NT functions.
.P
.cs R
.cs I
.nf
\fIE\fP "." \fIX\fP
.fi
.cs R
.cs I
.IP  \" nbsp #160 (pour Rman)
is the field selection operator. It denotes the value of field \fIX\fP
of the record value produced by evaluating expression \fIE\fP
(which must be of record type containing a field \fIX\fP).
.P
.cs R
.cs I
.nf
\fIE\fP "." "{" \fIX0\fP "=>" \fIE0\fP "," ... "," \fIXn\fP "=>" \fIEn\fP "}"
.fi
.cs R
.cs I
.IP  \" nbsp #160 (pour Rman)
is the field updating operator. It denotes the value of expression \fIE\fP
(which must be of record type containing the fields \fIX0\fP, ..., \fIXn\fP)
in which the fields \fIX0\fP, ..., \fIXn\fP have been replaced with the
values of expressions \fIE0\fP, ..., \fIEn\fP, respectively.
.P
.cs R
.cs I
.nf
\fIE1\fP "[" \fIE2\fP "]"
.fi
.cs R
.cs I
.IP  \" nbsp #160 (pour Rman)
is the array element operator, which denotes the element stored at the index
obtained by evaluating expression \fIE2\fP of the array obtained
by evaluating expression \fIE1\fP. Expression \fIE1\fP must be of array type
and expression \fIE2\fP must be of type \f4Nat\fP, which is predefined in
LOTOS NT.
.P
.cs R
.cs I
.nf
"(" \fIE\fP ")"
.fi
.cs R
.cs I
.IP  \" nbsp #160 (pour Rman)
has the same meaning as expression \fIE\fP. Parentheses are useful
for imposing an evaluation order of subexpressions different from the
order given by the associativity and precedence of operators.

.SS DECLARATIONS

Similarly to classical programming languages, PIC provides
mechanisms for declaring channels and data variables.
The syntax of PIC declarations (without initialization) is defined by
the following grammar:
.P
.cs 4 23
.cs I 23
.nf
\f4    \fIDECL\fP ::= \fIC\fP

          |  \fIX\fP ":" \fIT\fP\fR
.fi
.cs 4
.cs I
.P
Variable declarations have also a generalized form:

.nf
\fB    \fIX01\fP "," ... "," \fIX0m0\fP ":" \fIT0\fP
    "," ... ","
    \fIXn1\fP "," ... "," \fIXnmn\fP ":" \fITn\fP\fR
.fi

which declares the data variables \fIXi1\fP, ..., \fIXimi\fP of type \fITi\fP
for each 0 <= \fIi\fP <= \fIn\fP. This general form of declaration is
equivalent to the simplified form below, which will be used in the remainder
of this manual page:

.nf
\fB    \fIX01\fP ":" \fIT0\fP "," ... "," \fIX0m0\fP ":" \fIT0\fP
    "," ... ","
    \fIXn1\fP ":" \fITn\fP "," ... "," \fIXnmn\fP ":" \fITn\fP\fR
.fi

In the same way, variable declarations with initialization have the following
general form:

.nf
\fB    \fIX01\fP "," ... "," \fIX0m0\fP ":" \fIT0\fP ":=" \fIE0\fP
    "," ... ","
    \fIXn1\fP "," ... "," \fIXnmn\fP ":" \fITn\fP ":=" \fIEn\fP\fR
.fi

which declares the data variables \fIXi1\fP, ..., \fIXimi\fP of type \fITi\fP
and initializes them with the value of the expression \fIEi\fP for each
0 <= \fIi\fP <= \fIn\fP. This general form of declaration with initialization
is equivalent to the simplified form below, which will be used in the remainder
of this manual page:

.nf
\fB    \fIX01\fP ":" \fIT0\fP ":=" \fIE0\fP "," ... "," \fIX0m0\fP ":" \fIT0\fP ":=" \fIE0\fP
    "," ... ","
    \fIXn1\fP ":" \fITn\fP ":=" \fIEn\fP "," ... "," \fIXnmn\fP ":" \fITn\fP ":=" \fIEn\fP\fR
.fi

.SS BEHAVIOURS

Concurrent processes in PIC can manipulate data values in addition to
communication channels. Their behaviour is described using (generalizations
of) the polyadic pi-calculus operators and some data-handling constructs
inherited from LOTOS NT.
The syntax of PIC behaviours is defined by the following grammar:
.P
.cs 4 23
.cs I 23
.nf
\fB    \fIB\fP ::= "nil"

       |  \fIP\fP [ "(" \fIE0\fP "," ... "," \fIEn\fP ")" ]

       |  "tau" "." \fIB\fP

       |  "'" \fIC\fP [ "<" \fIE1\fP "," ... "," \fIEn\fP ">" ] "." \fIB\fP

       |  \fIC\fP [ "(" \fIDECL1\fP "," ... "," \fIDECLn\fP ")" ] "." \fIB\fP

       |  "[" \fIE\fP "]" \fIB\fP

       |  "!" [ \fIK\fP ] \fIB\fP

       |  "(" "new" \fIC0\fP "," ... "," \fICn\fP ")" \fIB\fP

       |  "var" \fIX0\fP ":" \fIT0\fP ":=" \fIE0\fP "," ... "," \fIXn\fP ":" \fITn\fP ":=" \fIEn\fP "in"
              \fIB\fP
          "end" "var"

       |  \fIB1\fP "+" \fIB2\fP

       |  \fIB1\fP "|" \fIB2\fP

       |  "(" \fIB\fP ")"\fR
.fi
.cs 4
.cs I
.P
Syntactically, all binary operators are left-associative.
Unary operators ("\f4.\fP", "\f4[\fP...\f4]\fP", "\f4!\fP", "new") have
the highest precedence, followed by "\f4|\fP", followed by "\f4+\fP".
.P
The semantics of a PIC behaviour \fIB\fP is given by a Labeled Transition
System (LTS) \fIM\fP = < \fIS\fP, \fIA\fP, \fIT\fP, \fIs0\fP >, where:
\fIS\fP is the set of states;
\fIA\fP is the set of actions;
\fIT\fP included in \fIS\fP * \fIA\fP * \fIS\fP is the transition relation;
and \fIs0\fP in \fIS\fP is the initial state.
An action \fIa\fP of \fIA\fP is either the silent action \fIi\fP
(this is the LOTOS NT notation, since the LTS denoting the semantics of
a PIC behaviour is produced from the corresponding LOTOS NT code generated
by \fBpic2lnt\fP), or it has the form \fIc\fP \fIv1\fP ... \fIvn\fP,
where \fIc\fP is a channel name and \fIv1\fP, ..., \fIvn\fP is a
(possibly empty) list of channel names or data values exchanged during
the rendezvous communication on channel \fIc\fP. This form of action
corresponds to the \fIearly\fP operational semantics of the pi-calculus
(see, e.g., [MP95,Par01]).
A transition (\fIs1\fP, \fIa\fP, \fIs2\fP) of \fIT\fP (also noted
\fIs1\fP-\fIa\fP->\fIs2\fP) means that the system can move from state \fIs1\fP
to state \fIs2\fP by performing action \fIa\fP.

The semantics of PIC behaviours is described informally below.
.P
.cs R
.cs I
.nf
"nil"
.fi
.cs R
.cs I
.IP  \" nbsp #160 (pour Rman)
is the empty behaviour (deadlock), which does not perform any action.
.P
.cs R
.cs I
.nf
\fIP\fP [ "(" \fIE0\fP "," ... "," \fIEn\fP ")" ]
.fi
.cs R
.cs I
.IP  \" nbsp #160 (pour Rman)
is a call to process \fIP\fP having as arguments the values of expressions
\fIE0\fP, ..., \fIEn\fP. The arguments must correspond in number and types
to the formal parameters given in the definition of process \fIP\fP
(see PROCESS DEFINITIONS below). This behaviour corresponds to the body
of process \fIP\fP, in which the formal parameters have been replaced by
the arguments \fIE0\fP, ..., \fIEn\fP, respectively.
.P
.cs R
.cs I
.nf
"tau" "." \fIB\fP
.fi
.cs R
.cs I
.IP  \" nbsp #160 (pour Rman)
is the silent prefix operator. It performs a transition labeled by the silent
action (noted \f4i\fP in LOTOS NT) and continues with the behaviour \fIB\fP.
.P
.cs R
.cs I
.nf
"'" \fIC\fP [ "<" \fIE1\fP "," ... "," \fIEn\fP ">" ] "." \fIB\fP
.fi
.cs R
.cs I
.IP  \" nbsp #160 (pour Rman)
is the emission on channel \fIC\fP of the channel names or values
\fIv1\fP, ..., \fIvn\fP obtained by evaluating the expressions
\fIE1\fP, ..., \fIEn\fP. It performs a transition labeled by an action
of the form \fIC\fP \fIv1\fP ... \fIvn\fP and continues with the
behaviour \fIB\fP.
.P
.cs R
.cs I
.nf
\fIC\fP [ "(" \fIDECL1\fP "," ... "," \fIDECLn\fP ")" ] "." \fIB\fP
.fi
.cs R
.cs I
.IP  \" nbsp #160 (pour Rman)
is the reception on channel \fIC\fP of \fIn\fP channel names or values
\fIv1\fP, ..., \fIvn\fP, the types of which must correspond to the
declarations \fIDECL1\fP, ..., \fIDECLn\fP, respectively. If a declaration
\fIDECLi\fP denotes a channel name \fICi\fP, then a channel name \fICi'\fP
is expected to be received at position \fIi\fP and is stored in \fICi\fP.
If a declaration \fIDECLj\fP denotes a variable \fIXi\fP of type \fITi\fP,
then a value \fIvi\fP of type \fITi\fP is expected to be received at position
\fIj\fP and is stored in \fIXi\fP. It performs a transition labeled by an
action of the form \fIC\fP \fIv1\fP ... \fIvn\fP and continues with the
behaviour \fIB\fP.
.sp
All channel names and variables declared by \fIDECL1\fP, ..., \fIDECLn\fP are
visible in \fIB\fP.
.P
.cs R
.cs I
.nf
"[" \fIE\fP "]" \fIB\fP
.fi
.cs R
.cs I
.IP  \" nbsp #160 (pour Rman)
is the guard operator. It has the behaviour \fIB\fP if the boolean condition
\fIE\fP evaluates to true and is equivalent to "nil" otherwise.
The guard operator of PIC accepts any boolean expression \fIE\fP, whereas
the guard operator of standard pi-calculus allows only the equality test
between channel names. The "if-then-else" conditional behaviour can be
simulated by using the selection operator and mutually exclusive guard
operators as follows [Par01]:
.sp
.cs 4 23
.cs I 23
.nf
\fB    if E then B1 else B2 = [ E ] B1 + [ not (E) ] B2\fR
.fi
.cs 4
.cs I
.P
.cs R
.cs I
.nf
"!" [ \fIK\fP ] \fIB\fP
.fi
.cs R
.cs I
.IP  \" nbsp #160 (pour Rman)
is the bounded replication operator. It is a shorthand notation for the
behaviour "\fIB\fP | ... | \fIB\fP", where \fIB\fP occurs \fIK\fP times,
where \fIK\fP>=1 is a constant value of type \f4Nat\fP. If \fIK\fP is
absent, it has the default value 1, meaning that the bounded replication
operator is a shorthand for \fIB\fP.
.P
.cs R
.cs I
.nf
"(" "new" \fIC0\fP "," ... "," \fICn\fP ")" \fIB\fP
.fi
.cs R
.cs I
.IP  \" nbsp #160 (pour Rman)
is the channel creation operator. It defines the "fresh" channel names
\fIC0\fP, ..., \fICn\fP, which are visible in \fIB\fP. These channel names 
are \fIprivate\fP, meaning that they cannot be used for communication between
\fIB\fP and another behaviour \fIB'\fP unless they are communicated by \fIB\fP
to \fIB'\fP on some other channel name known by both \fIB\fP and \fIB'\fP
(in this case, a channel \fICk\fP sent by \fIB\fP to \fIB'\fP becomes visible
in \fIB'\fP by scope extrusion).
.P
.cs R
.cs I
.nf
"var" \fIX0\fP ":" \fIT0\fP ":=" \fIE0\fP "," ... "," \fIXn\fP ":" \fITn\fP ":=" \fIEn\fP "in"
    \fIB\fP
"end" "var"
.fi
.cs R
.cs I
.IP  \" nbsp #160 (pour Rman)
is the variable definition operator. It declares the variables \fIX0\fP, ...,
\fIXn\fP and initializes them with the result of evaluating the expressions
\fIE0\fP, ..., \fIEn\fP, which must be of type \fIT0\fP, ..., \fITn\fP,
respectively.
.sp
The variables \fIX0\fP, ..., \fIXn\fP are visible in \fIB\fP.
.P
.cs R
.cs I
.nf
\fIB1\fP "+" \fIB2\fP
.fi
.cs R
.cs I
.IP  \" nbsp #160 (pour Rman)
is the choice operator. It performs either an action of \fIB1\fP and
continues with the behaviour of \fIB1\fP, or an action of \fIB2\fP and 
continues with the behaviour of \fIB2\fP.
.P
.cs R
.cs I
.nf
\fIB1\fP "|" \fIB2\fP
.fi
.cs R
.cs I
.IP  \" nbsp #160 (pour Rman)
is the parallel composition operator. It produces the interleaved execution
of the behaviours \fIB1\fP and \fIB2\fP according to the early operational
semantics of pi-calculus [MP95,Par01].
.P
.cs R
.cs I
.nf
"(" \fIB\fP ")"
.fi
.cs R
.cs I
.IP  \" nbsp #160 (pour Rman)
has the same meaning as behaviour \fIB\fP. Parentheses are useful
for imposing an evaluation order of behaviours different from the
order given by the associativity and precedence of operators.

.SS PROCESS DEFINITIONS

Concurrent behaviours can be encapsulated into processes parameterized
by channel names and data values, which can be instantiated multiple
times in a PIC specification. The syntax of PIC process definitions is
defined by the following grammar:
.P
.cs 4 23
.cs I 23
.nf
\fB    \fIPROC\fP ::= \fIP\fP [ "(" \fIDECL1\fP "," ... "," \fIDECLn\fP ")" ] "=" \fIB\fP\fR
.fi
.cs 4
.cs I
.P
This defines the process \fIP\fP, equipped with the optional (channel or
variable) parameters declared by \fIDECL1\fP, ..., \fIDECLn\fP. The parameters
are visible in the behaviour \fIB\fP, which is the body of process \fIP\fP.
A process \fIP\fP without parameters can be called using the syntax \fIP\fP
or \fIP\fP(). Processes can be mutually recursive.
.sp
Note: The current version of \fBpic2lnt\fP handles only PIC specifications
satisfying the \fIfinite control\fP property [Dam94], which forbids recursive
process calls through the parallel composition operator. This hypothesis was
used in the translation from polyadic pi-calculus to LOTOS NT [MS10] and was
maintained for the translation from PIC to LOTOS NT.
.sp
For example, the process definition below does not satisfy the finite
control property:
.P
.cs 4 23
.cs I 23
.nf
\fB    P = P | (a . nil)\fR
.fi
.cs 4
.cs I
.P
Note: Recursive process calls through the channel creation operator are
permitted. For instance, the following process definition is handled by
\fBpic2lnt\fP:
.P
.cs 4 23
.cs I 23
.nf
\fB    Q = (new b) (b . Q)\fR
.fi
.cs 4
.cs I
.P
A call to process Q yields an infinite LTS, because each recursive
call will generate a new transition labeled by a fresh channel name
"\f4b (n)\fP" for n >= 0. Therefore, trying to generate the LTS of
a call to process Q will exhaust the memory available on the host computer.

.SS SPECIFICATION

A PIC specification consists of a list of process definitions possibly preceded
by the declaration of external LOTOS NT modules.
The syntax of a PIC specification is defined by the grammar below:
.P
.cs 4 23
.cs I 23
.nf
\fB    \fISPEC\fP ::= [ "library" \fIM0\fP "," ..."," \fIMm\fP "end" "library" ]
             \fIPROC0\fP ... \fIPROCn\fP\fR
.fi
.cs 4
.cs I
.P
If present, the module identifiers \fIM0\fP, ..., \fIMm\fP must correspond to
LOTOS NT files named \fIM0\fP\f4.lib\fP, ..., \fIMm\fP\f4.lib\fP present in
the current directory (letter case is not significant).
The LOTOS NT modules \fIM0\fP, ..., \fIMm\fP must define all the data types and
functions used in the process definitions \fIPROC0\fP, ..., \fIPROCn\fP.
If the "library" clause is absent, the PIC specification is dataless, i.e.,
it contains only polyadic pi-calculus agent definitions.

.SS LABELED TRANSITION SYSTEM

For technical reasons, the translation from PIC to LOTOS NT adds extra
information on the actions (transition labels) of the LTS w.r.t. the original
early operational semantics of the pi-calculus [MP95,Par01].
This extra information regards two aspects:
.IP -
\fIChannel names.\fP
All channel names occurring in the PIC specification are converted to
uppercase. To bring the PIC behaviours to normal form (i.e., to avoid that
the same channel name has both free and bound occurrences in the same PIC
behaviour), each channel name is postfixed by "_\fIk\fP", where \fIk\fP is
a natural number unique for each channel name. For example, a channel name
"a" present in the PIC specification is translated into "A_\fIk\fP" for some
\fIk\fP. Public (resp. private) channel names are encapsulated into a "PUB"
(resp. "PRIV") constructor. Moreover, private channel names have a natural
number parameter allowing to generate fresh private channel names during the
execution of the PIC specification. For example, a public channel name "b"
present in the PIC specification will be translated into "PUB (B_\fIk\fP)"
for some \fIk\fP, and a private channel name "c" will be translated into
"PRIV (C_\fIk\fP (\fIj\fP))" for some \fIk\fP and \fIj\fP.
.IP -
\fICommunication.\fP
All visible actions in the LTS take place on a special channel named "PUBLIC".
The first value exchanged on each action is the name of the PIC channel on
which the communication takes place in the PIC specification. The other values
are those communicated by the PIC action. In addition, there is a last boolean
value added on each LTS action to indicate whether the corresponding PIC
action is an emission (value TRUE) or a reception (value FALSE). According to
the LOTOS NT and LOTOS semantics conventions, all values exchanged on channel
"PUBLIC" are preceded by '!'.
For example, an emission of the form "'a < b, c, 3 of Nat >",
where \f4a\fP, \f4b\fP are public channels
and \f4c\fP is a private channel (i.e., defined by a "new" operator), will
yield an LTS transition labeled by the following action:
.sp
.nf
  PUBLIC !PUB (A_0) !PUB (B_0) !PRIV (C_1 (1)) !3 !TRUE
.fi
.P
Starting from a PIC specification \fIfilename\fP\f4.pic\fR, the corresponding
LTS can be produced by applying \fBpic2lnt\fP and then generating the LTS
\fIfilename\fP\f4.bcg\fR, encoded in the BCG file format, of the resulting
LOTOS NT specification. This LTS can be made compatible with the original
pi-calculus semantics by renaming its actions using
.BR bcg_labels (LOCAL)
and the renaming files present in $PIC/incl as follows:
.sp
.nf
  bcg_labels -rename -multiple $PIC/incl/pic_renaming_1.ren
  -rename -multiple $PIC/incl/pic_renaming_2.ren
  -rename $PIC/incl/pic_renaming_3.ren
  \fIfilename\fP\f4.bcg\fR
.fi
.P
Note: this renaming does not delete the suffixes "_\fIk\fP" of the channel
names, in order to facilitate the understanding of the PIC behaviours that
are not in normal form (i.e., contain both free and bound occurrences of
the same channel name(s)).

.SS SYNTACTIC COMPATIBILITY

For compatibility with the Mobility Workbench (MWB) [VM94],
the alternative syntax below is also accepted by \fBpic2lnt\fP for the
polyadic pi-calculus constructs:
.P
.cs 4 23
.cs I 23
.nf
+-----------------+-----------------------------------------+
|   pi-calculus   |        Mobility Workbench (MWB)         |
|    construct    |                 syntax                  |
+=================+=========================================+
| empty           | "0"                                     |
+-----------------+-----------------------------------------+
| silent prefix   | "t" "." \fIB\fP                               |
+-----------------+-----------------------------------------+
| emission        | "'" \fIC\fP "<" \fIC1\fP ... \fICn\fP ">" "." \fIB\fP           |
+-----------------+-----------------------------------------+
| reception       | \fIC\fP "(" \fIC1\fP ... \fICn\fP ")" "." \fIB\fP               |
+-----------------+-----------------------------------------+
| channel         | "(" "new" \fIC0\fP ... \fICn\fP ")" \fIB\fP               |
| creation        | "(" "^" \fIC0\fP ... \fICn\fP ")" \fIB\fP                 |
|                 | "(" "^" \fIC0\fP "," ... "," \fICn\fP ")" \fIB\fP         |
+-----------------+-----------------------------------------+
| process (agent) | "agent" \fIP\fP [ "(" \fIC0\fP "," ... "," \fICn\fP ")" ] |
| definition      | "=" \fIB\fP                                   |
+-----------------+-----------------------------------------+
.fi
.cs 4
.cs I

.SH OPERANDS
.PD 0
.TP 25
.IB filename .pic
PIC specification (input)
.TP
.IB module .lib
LOTOS NT code for data types and functions (input)
.TP
.IB filename .lnt
LOTOS NT code (output)

.SH ENVIRONMENT VARIABLES
.PD 0
.TP 25
.B $PIC
Name of the directory where \fBpic2lnt\fP is installed.

.SH FILES
.PD 0
.TP 25
.B $PIC/incl/pic_renaming_{1,2,3}.ren
Renaming files for converting the LTS labels according to the original
semantics of the polyadic pi-calculus.
.PD 0
.TP 25
.B $PIC/incl/pic2lnt_dyn.tnt
Auxiliary file necessary for compiling the LOTOS NT specification
\fIfilename\fP\f4.lnt\fR produced as output by \fBpic2lnt\fP.
To compile \fIfilename\fP\f4.lnt\fR using
.BR lnt2lotos (LOCAL)
or
.BR lnt.open (LOCAL),
the file $PIC/incl/pic2lnt_dyn.tnt should be copied in the current
directory and renamed into \fIfilename\fP\f4.tnt\fR.

.SH EXIT STATUS

If the translation was successful the exit status is 0, even if warnings were
issued during the execution. If any error occurred during translation, the exit
status is 1.

.SH BIBLIOGRAPHY

.TP
[Dam94]
M. Dam.
"Model Checking Mobile Processes."
Research Report RR 94:1, Swedish Institute of Computer Science,
Kista, Sweden, 1994.

.TP
[MP95]
U. Montanari and M. Pistore.
"Checking Bisimilarity for Finitary Pi-Calculus."
Proceedings of CONCUR'95, LNCS v. 962, p. 42-56, 1995.

.TP
[MS10]
R. Mateescu and G. Salaun.
"Translating Pi-Calculus to LOTOS NT."
Proceedings of IFM'10, LNCS v. 6396, p. 229-244, 2010.

.TP
[Par01]
J. Parrow.
"An Introduction to the Pi-Calculus."
Handbook of Process Algebra, chap. 8, p. 479-544, North-Holland, 2001.

.TP
[VM94]
B. Victor and F. Moller.
"The Mobility Workbench: A Tool for the Pi-Calculus."
Proceedings of CAV'94, LNCS v. 818, p. 428-440, 1994.

.TP
[MPW92]
R. Milner, J. Parrow, and D. Walker.
"A Calculus of Mobile Processes."
Information and Computation 100(1):1-77, 1992.

.SH AUTHORS
Gwen Salaun (Grenoble INP) and Radu Mateescu (INRIA Grenoble - Rhone\-Alpes).

.SH SEE ALSO
.BR lnt.open (LOCAL),
.BR bcg (LOCAL),
.BR bcg_labels (LOCAL).
For a complete description of LOTOS NT, see the \fBlnt2lotos\fP
reference manual.

.SH BUGS

Please report any mistranslations or other problems with \fBpic2lnt\fP to 
cadp@inria.fr

